"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel
from typing_extensions import TypedDict


class ForterAntiFraudOptionsCartItemBeneficiaryPhoneTypedDict(TypedDict):
    phone: str
    r"""The phone number of the beneficiary."""


class ForterAntiFraudOptionsCartItemBeneficiaryPhone(BaseModel):
    phone: str
    r"""The phone number of the beneficiary."""
