"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from gr4vy.types import BaseModel
from typing_extensions import TypedDict


class ReportExecutionURLTypedDict(TypedDict):
    url: str
    r"""A signed URL to download the report execution file."""
    expires_at: datetime
    r"""The date and time when the download URL expires."""


class ReportExecutionURL(BaseModel):
    url: str
    r"""A signed URL to download the report execution file."""

    expires_at: datetime
    r"""The date and time when the download URL expires."""
